#define pastix_int_t     long
#define pastix_uint_t    unsigned long
#define MPI_PASTIX_INT   MPI_LONG
#define pastix_float_t   double
#define MPI_PASTIX_FLOAT MPI_DOUBLE
#define FLOAT            pastix_float_t
#define INT              pastix_int_t
#define UINT             pastix_uint_t
/* Copyright 2008 BORDEAUX I UNIVERSITY & INRIA 
**
** This file is part of the PaStiX parallel sparse matrix package.
**
** This software is governed by the CeCILL-C license under French law
** and abiding by the rules of distribution of free software. You can
** use, modify and/or redistribute the software under the terms of the
** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
** URL: "http://www.cecill.info".
** 
** As a counterpart to the access to the source code and rights to copy,
** modify and redistribute granted by the license, users are provided
** only with a limited warranty and the software's author, the holder of
** the economic rights, and the successive licensors have only limited
** liability.
** 
** In this respect, the user's attention is drawn to the risks associated
** with loading, using, modifying and/or developing or reproducing the
** software by the user in light of its specific status of free software,
** that may mean that it is complicated to manipulate, and that also
** therefore means that it is reserved for developers and experienced
** professionals having in-depth computer knowledge. Users are therefore
** encouraged to load and test the software's suitability as regards
** their requirements in conditions enabling the security of their
** systems and/or data to be ensured and, more generally, to use and
** operate it in the same conditions as regards security.
** 
** The fact that you are presently reading this means that you have had
** knowledge of the CeCILL-C license and that you accept its terms.
*/
/** Structure PaStiX pour une instance du solveur */
struct pastix_data_t;
typedef struct pastix_data_t pastix_data_t;
#ifdef   __cplusplus
#ifndef COMPLEXFLOAT_
#define COMPLEXFLOAT_
struct{float realnum; float imagnum;} ComplexFloat_;
#endif
#ifndef COMPLEXDOUBLE_
#define COMPLEXDOUBLE_
struct{double realnum; double imagnum;} ComplexDouble_;
#endif
#define  COMPLEX struct ComplexFloat_
#define  DCOMPLEX struct ComplexDouble_
#else
#define  COMPLEX float complex
#define  DCOMPLEX double complex
#endif

/* Only works if no FLOAT in args */
#if (defined _COMPLEX_H || defined _H_COMPLEX)
#define MULTIPLE_TYPE_DEFINE(functype, funcname, funcargs) \
  functype s_ ## funcname funcargs;			   \
  functype d_ ## funcname funcargs;			   \
  functype c_ ## funcname funcargs;			   \
  functype z_ ## funcname funcargs;
#else
#define MULTIPLE_TYPE_DEFINE(functype, funcname, funcargs) \
  functype s_ ## funcname funcargs;			   \
  functype d_ ## funcname funcargs;
#endif
#if (defined _COMPLEX_H || defined _H_COMPLEX)
#define MULTIPLE_TYPE_DEFINE_F(functype,			\
			       funcname,			\
			       funcargs_s,			\
			       funcargs_d,			\
			       funcargs_c,			\
			       funcargs_z)			\
  functype s_ ## funcname funcargs_s;				\
  functype d_ ## funcname funcargs_d;				\
  functype c_ ## funcname funcargs_c;				\
  functype z_ ## funcname funcargs_z;
#else
#define MULTIPLE_TYPE_DEFINE_F(functype,			\
			       funcname,			\
			       funcargs_s,			\
			       funcargs_d,			\
			       funcargs_c,			\
			       funcargs_z)			\
  functype s_ ## funcname funcargs_s;				\
  functype d_ ## funcname funcargs_d;
#endif

#if (defined FLOAT)
/*
  Function: pastix

  Computes one to all steps of the resolution of Ax=b linear system, using direct methods.
  
  The matrix is given in CSC format.

  Parameters:
    pastix_data - Data used for a step by step execution.
    pastix_comm - MPI communicator which compute the resolution.
    n           - Size of the system.
    colptr      - Tabular containing the start of each column in row and avals tabulars.
    row         - Tabular containing the row number for each element sorted by column.
    avals       - Tabular containing the values of each elements sorted by column.
    perm        - Permutation tabular for the renumerotation of the unknowns.
    invp        - Reverse permutation tabular for the renumerotation of the unknowns.
    b           - Right hand side vector(s).
    rhs         - Number of right hand side vector(s).
    iparm       - Integer parameters given to pastix.
    dparm       - Double parameters given to pstix.

  About: Example

    from file <simple.c> :

    > /\*******************************************\/
    > /\*    Check Matrix format                  *\/
    > /\*******************************************\/
    > /\*                                      
    >  * Matrix needs :
    >  *    - to be in fortran numbering
    >  *    - to have only the lower triangular part in symmetric case
    >  *    - to have a graph with a symmetric structure in unsymmetric case
    >  *\/
    > pastix_checkMatrix(MPI_COMM_WORLD, verbosemode, 
    >       	     (MTX_ISSYM(type) ? API_SYM_YES : API_SYM_NO),  API_YES,
    >       	     ncol, &colptr, &rows, &values, NULL);
    > 
    > /\*******************************************\/
    > /\* Initialize parameters to default values *\/
    > /\*******************************************\/
    > iparm[IPARM_MODIFY_PARAMETER] = API_NO;
    > pastix(&pastix_data, MPI_COMM_WORLD, 
    >        ncol, colptr, rows, values, 
    >        perm, invp, rhs, 1, iparm, dparm);

    > /\*******************************************\/
    > /\*       Customize some parameters         *\/
    > /\*******************************************\/
    > iparm[IPARM_THREAD_NBR] = nbthread;
    > if (MTX_ISSYM(type)) {
    >   :wiparm[IPARM_SYM]           = API_SYM_YES;
    >   iparm[IPARM_FACTORIZATION] = API_FACT_LDLT;
    > }
    > else{
    >   iparm[IPARM_SYM]           = API_SYM_NO;
    >   iparm[IPARM_FACTORIZATION] = API_FACT_LU;
    > }
    > iparm[IPARM_START_TASK]          = API_TASK_ORDERING;
    > iparm[IPARM_END_TASK]            = API_TASK_CLEAN;

    > /\*******************************************\/
    > /\*           Save the rhs                  *\/
    > /\*    (it will be replaced by solution)    *\/
    > /\*******************************************\/
    > rhssaved = malloc(ncol*sizeof(pastix_float_t));
    > memcpy(rhssaved, rhs, ncol*sizeof(pastix_float_t));

    > /\*******************************************\/
    > /\*           Call pastix                   *\/
    > /\*******************************************\/
    > perm = malloc(ncol*sizeof(pastix_int_t));
    > invp = malloc(ncol*sizeof(pastix_int_t));

    > pastix(&pastix_data, MPI_COMM_WORLD, 
    >	 ncol, colptr, rows, values,
    >	 perm, invp, rhs, 1, iparm, dparm);
*/
void pastix(pastix_data_t **pastix_data, MPI_Comm pastix_comm, INT n, INT *colptr, INT *row, 
	    FLOAT *avals, INT *perm, INT *invp, FLOAT *b, INT rhs, INT *iparm, double *dparm);
#endif
MULTIPLE_TYPE_DEFINE_F(void, pastix,
		       ( pastix_data_t **pastix_data, MPI_Comm pastix_comm, INT n, INT *colptr, INT *row, 
			 float *avals, INT *perm, INT *invp, float *b, INT rhs, INT *iparm, double *dparm),

		       ( pastix_data_t **pastix_data, MPI_Comm pastix_comm, INT n, INT *colptr, INT *row, 
			 double *avals, INT *perm, INT *invp, double *b, INT rhs, INT *iparm, double *dparm),

		       ( pastix_data_t **pastix_data, MPI_Comm pastix_comm, INT n, INT *colptr, INT *row, 
			 COMPLEX *avals, INT *perm, INT *invp, COMPLEX *b, INT rhs, INT *iparm, double *dparm),

		       ( pastix_data_t **pastix_data, MPI_Comm pastix_comm, INT n, INT *colptr, INT *row, 
			 DCOMPLEX *avals, INT *perm, INT *invp, DCOMPLEX *b, INT rhs, INT *iparm, double *dparm))
#if (defined FLOAT)
/** Function: dpastix

    Computes one to all steps of the resolution of Ax=b linear system, using direct methods.
    Here the matrix is given distributed.

    The matrix is given in CSCD format.

    Parameters: 
       pastix_data - Data used for a step by step execution.
       pastix_comm - MPI communicator which compute the resolution.
       n           - Size of the system.
       colptr      - Tabular containing the start of each column in row and avals tabulars.
       row         - Tabular containing the row number for each element sorted by column.
       avals       - Tabular containing the values of each elements sorted by column.
       loc2glob    - Global column number of the local columns.
       perm        - Permutation tabular for the renumerotation of the unknowns.
       invp        - Reverse permutation tabular for the renumerotation of the unknowns.
       b           - Right hand side vector(s).
       rhs         - Number of right hand side vector(s).
       iparm       - Integer parameters given to pastix.
       dparm       - Double parameters given to pstix.
 */
void dpastix(pastix_data_t **pastix_data, MPI_Comm pastix_comm, INT n, INT *colptr, INT *row, 
	     FLOAT *avals, INT * loc2glob, INT *perm, INT *invp, FLOAT *b, INT rhs, INT *iparm, double *dparm);
#endif
MULTIPLE_TYPE_DEFINE_F(void, dpastix,
		       (pastix_data_t **pastix_data, MPI_Comm pastix_comm, INT n, INT *colptr, INT *row, 
			float *avals, INT * loc2glob, INT *perm, INT *invp, float *b, INT rhs, INT *iparm, 
			double *dparm),
		       
		       (pastix_data_t **pastix_data, MPI_Comm pastix_comm, INT n, INT *colptr, INT *row, 
			double *avals, INT * loc2glob, INT *perm, INT *invp, double *b, INT rhs, INT *iparm, 
			double *dparm),

		       (pastix_data_t **pastix_data, MPI_Comm pastix_comm, INT n, INT *colptr, INT *row, 
			COMPLEX *avals, INT * loc2glob, INT *perm, INT *invp, COMPLEX *b, INT rhs, INT *iparm, 
			double *dparm),

		       (pastix_data_t **pastix_data, MPI_Comm pastix_comm, INT n, INT *colptr, INT *row, 
			DCOMPLEX *avals, INT * loc2glob, INT *perm, INT *invp, DCOMPLEX *b, INT rhs, INT *iparm, 
			double *dparm))
#if (defined FLOAT)
/*
  Function: pastix_setBind

  Set bindtab in pastix_data, it gives for each thread the CPU to bind in to.
  bindtab follows this organisation :

  bindtab[threadnum] = cpu to set thread threadnum.

  Parameters:
    pastix_data - Structure de donne pour l'utilisation step by step
    thrdnbr     - Nombre de threads / Taille du tableau
    bindtab     - Tableau de correspondance entre chaque thread et coeur de la machine
*/
void pastix_setBind(pastix_data_t *pastix_data, int thrdnbr, int *bindtab);
#endif
MULTIPLE_TYPE_DEFINE(void, pastix_setBind,
		     (pastix_data_t *pastix_data, int thrdnbr, int *bindtab))
#if (defined FLOAT)
/*
  Function: pastix_checkMatrix 

  Check the matrix :
    - Renumbers in Fortran numerotation (base 1) if needed (base 0)
    - Can scale the matrix if compiled with -DMC64 -DSCALING (untested)
    - Checks the symetry of the graph in non symmetric mode.
      With non distributed matrices, with flagcor == API_YES,
      tries to correct the matrix.
    - sort the CSC.

  Parameters:
    pastix_comm -  PaStiX MPI communicator
    verb        -  Level of prints (API_VERBOSE_[NOT|NO|YES])
    flagsym     -  Indicate if the given matrix is symetric (API_SYM_YES or API_SYM_NO)
    flagcor     -  Indicate if we permit the function to reallocate the matrix.
    n           -  Size of the matrix.
    colptr      -  First element of each row in *row* and *avals*.
    row         -  Row of each element of the matrix.
    avals       -  Value of each element of the matrix.
    loc2glob    -  Global column number of local columns.
    dof         - Number of degrees of freedom.
*/
int pastix_checkMatrix(MPI_Comm pastix_comm, int verb, int flagsym, int flagcor,
		       INT n, INT **colptr, INT **row, FLOAT **avals, INT **loc2glob, int dof);
#endif
MULTIPLE_TYPE_DEFINE_F(int, pastix_checkMatrix,
		       (MPI_Comm pastix_comm, int verb, int flagsym, int flagcor,
			INT n, INT **colptr, INT **row, float **avals, INT **loc2glob, int dof),

		       (MPI_Comm pastix_comm, int verb, int flagsym, int flagcor,
			INT n, INT **colptr, INT **row, double **avals, INT **loc2glob, int dof),

		       (MPI_Comm pastix_comm, int verb, int flagsym, int flagcor,
			INT n, INT **colptr, INT **row, COMPLEX **avals, INT **loc2glob, int dof),
		       
		       (MPI_Comm pastix_comm, int verb, int flagsym, int flagcor,
			INT n, INT **colptr, INT **row, DCOMPLEX **avals, INT **loc2glob, int dof))

#if (defined FLOAT)
/*
  Function: pastix_getLocalNodeNbr

  Return the node number in the new distribution computed by blend.
  Needs blend to be runned with pastix_data before.

  Parameters:
    pastix_data - Data used for a step by step execution.

  Returns:
    Number of local nodes/columns in new distribution.
 */
INT pastix_getLocalNodeNbr(pastix_data_t ** pastix_data);
#endif
MULTIPLE_TYPE_DEFINE(INT, pastix_getLocalNodeNbr, (pastix_data_t ** pastix_data))

#if (defined FLOAT)
/*
  Function: pastix_getLocalNodeLst

  Fill in nodelst with the list of local nodes/clumns.
  Needs nodelst to be allocated with nodenbr*sizeof(pastix_int_t), 
  where nodenbr has been computed by <pastix_getLocalNodeNbr>. 

  Parameters:
    pastix_data - Data used for a step by step execution.
    nodelst     - An array where to write the list of local nodes/columns.
 */
int pastix_getLocalNodeLst(pastix_data_t ** pastix_data, INT * nodelst);
#endif
MULTIPLE_TYPE_DEFINE(int, pastix_getLocalNodeLst,(pastix_data_t ** pastix_data, INT * nodelst))

#if (defined FLOAT)
/*
  Function: pastix_getLocalUnknownNbr

  Return the unknown number in the new distribution computed by blend.
  Needs blend to be runned with pastix_data before.

  Parameters:
    pastix_data - Data used for a step by step execution.

  Returns:
    Number of local unknowns/columns in new distribution.
 */
INT pastix_getLocalUnknownNbr(pastix_data_t ** pastix_data);
#endif
MULTIPLE_TYPE_DEFINE(INT, pastix_getLocalUnknownNbr, (pastix_data_t ** pastix_data))
#if (defined FLOAT)
/*
  Function: pastix_getLocalUnknownLst

  Fill in unknownlst with the list of local unknowns/clumns.
  Needs unknownlst to be allocated with unknownnbr*sizeof(pastix_int_t), 
  where unknownnbr has been computed by <pastix_getLocalUnknownNbr>. 

  Parameters:
    pastix_data - Data used for a step by step execution.
    unknownlst     - An array where to write the list of local unknowns/columns.
 */
int pastix_getLocalUnknownLst(pastix_data_t ** pastix_data, INT * unknownlst);
#endif
MULTIPLE_TYPE_DEFINE(int, pastix_getLocalUnknownLst, (pastix_data_t ** pastix_data, INT * unknownlst))

#if (defined FLOAT)
/*
  Function: pastix_setSchurUnknownList

  Set the list of unknowns to isolate at the end 
  of the matrix via permutations.

  Has to be called if using IPARM_SCHUR = API_YES.

  Parameters:
    pastix_data - Data used for a step by step execution.
    n           - Number of unknowns.
    list        - List of unknowns.
*/
int pastix_setSchurUnknownList(pastix_data_t * pastix_data, 
			       INT  n,
			       INT *list);
#endif
MULTIPLE_TYPE_DEFINE(int, pastix_setSchurUnknownList, (pastix_data_t * pastix_data, 
						       INT  n,
						       INT *list))

#ifdef FLOAT
/*
  Function: pastix_getSchurLocalNodeNbr

  Compute the number of nodes in the local part of the Schur.

  Parameters:
    pastix_data - Common data structure for PaStiX calls.
    nodeNbr     - (out) Number of nodes in schur (local).

  Returns:
    NO_ERR      - For the moment

  TODO: Error management.
*/
int pastix_getSchurLocalNodeNbr(pastix_data_t * pastix_data, INT * nodeNbr);
#endif
MULTIPLE_TYPE_DEFINE(int, pastix_getSchurLocalNodeNbr, (pastix_data_t * pastix_data, INT * nodeNbr))

#ifdef FLOAT
/*
  Function: pastix_getSchurLocalUnkownNbr

  Compute the number of unknowns in the local part of the Schur.

  Parameters:
    pastix_data - Common data structure for PaStiX calls.
    unknownNbr  - (out) Number of unknowns in schur (local).

  Returns:
    NO_ERR      - For the moment

  TODO: Error management.
*/
int pastix_getSchurLocalUnkownNbr(pastix_data_t * pastix_data, INT * unknownNbr);
#endif
MULTIPLE_TYPE_DEFINE(int, pastix_getSchurLocalUnkownNbr, (pastix_data_t * pastix_data, INT * unknownNbr))

#ifdef FLOAT
/*
  Function: pastix_getSchurLocalNodeList

  Compute the list of nodes in the local part of the Schur.

  Parameters:
    pastix_data - Common data structure for PaStiX calls.
    nodes     - (out) Nodes in schur (local).

  Returns:
    NO_ERR      - For the moment

  TODO: Error management.
*/
int pastix_getSchurLocalNodeList(pastix_data_t * pastix_data, INT * nodes);
#endif
MULTIPLE_TYPE_DEFINE(int, pastix_getSchurLocalNodeList, (pastix_data_t * pastix_data, INT * nodes))

#ifdef FLOAT
/*
  Function: pastix_getSchurLocalUnkownList

  Compute the list of unknowns in the local part of the Schur.

  Parameters:
    pastix_data - Common data structure for PaStiX calls.
    unknowns    - (out) Unknowns in schur (local).

  Returns:
    NO_ERR      - For the moment

  TODO: Error management.
*/
int pastix_getSchurLocalUnknownList(pastix_data_t * pastix_data, INT * unknowns);
#endif
MULTIPLE_TYPE_DEFINE(int, pastix_getSchurLocalUnknownList, (pastix_data_t * pastix_data, INT * unknowns))

#ifdef FLOAT
/*
  Function: pastix_setSchurArray

  Give user memory area to store schur in PaStiX.

  Parameters:
    pastix_data - Common data structure for PaStiX calls.
    array       - Memory area to store the schur.

  Returns:
    NO_ERR      - For the moment

  TODO: Error management.
*/
int pastix_setSchurArray(pastix_data_t * pastix_data, FLOAT * array);
#endif
MULTIPLE_TYPE_DEFINE_F(int, pastix_setSchurArray,
		       (pastix_data_t * pastix_data, float * array),
		       (pastix_data_t * pastix_data, double * array),
		       (pastix_data_t * pastix_data, COMPLEX * array),
		       (pastix_data_t * pastix_data, DCOMPLEX * array))

#if (defined FLOAT)
/*
  Function: pastix_getSchur

  Get the Schur complement from PaStiX.
  
  Schur complement is a dense block in a 
  column scheme.
  
  Parameters:
    pastix_data - Data used for a step by step execution.
    schur - Array to fill-in with Schur complement.
    
*/
int pastix_getSchur(pastix_data_t * pastix_data, 
		    FLOAT * schur);

#endif
MULTIPLE_TYPE_DEFINE_F(int, pastix_getSchur, 
		       (pastix_data_t * pastix_data, float * schur),
		       (pastix_data_t * pastix_data, double * schur),
		       (pastix_data_t * pastix_data, COMPLEX * schur),
		       (pastix_data_t * pastix_data, DCOMPLEX* schur))

#if (defined FLOAT)
/*
  Function: pastix_initParam

  sets default parameters for iparm and dparm

  Parameters:
    iparm - tabular of IPARM_SIZE integer parameters.
    dparm - tabular of DPARM_SIZE double parameters.
*/
void pastix_initParam(INT    *iparm,
		      double *dparm);
#endif
MULTIPLE_TYPE_DEFINE(void, pastix_initParam, (INT    *iparm,
					      double *dparm))
/* Copyright 2008 BORDEAUX I UNIVERSITY & INRIA 
**
** This file is part of the PaStiX parallel sparse matrix package.
**
** This software is governed by the CeCILL-C license under French law
** and abiding by the rules of distribution of free software. You can
** use, modify and/or redistribute the software under the terms of the
** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
** URL: "http://www.cecill.info".
** 
** As a counterpart to the access to the source code and rights to copy,
** modify and redistribute granted by the license, users are provided
** only with a limited warranty and the software's author, the holder of
** the economic rights, and the successive licensors have only limited
** liability.
** 
** In this respect, the user's attention is drawn to the risks associated
** with loading, using, modifying and/or developing or reproducing the
** software by the user in light of its specific status of free software,
** that may mean that it is complicated to manipulate, and that also
** therefore means that it is reserved for developers and experienced
** professionals having in-depth computer knowledge. Users are therefore
** encouraged to load and test the software's suitability as regards
** their requirements in conditions enabling the security of their
** systems and/or data to be ensured and, more generally, to use and
** operate it in the same conditions as regards security.
** 
** The fact that you are presently reading this means that you have had
** knowledge of the CeCILL-C license and that you accept its terms.
*/
/*
  header: api.h

  Header file containing constants used in PaStiX and provided to users.

  Authors:
    Mathieu Faverge - faverge@labri.fr
    Xavier   Lacoste - lacoste@labri.fr
    Pierre Ramet     - ramet@labri.fr
 
 */

/* Acces au tableau iparm*/
/* 
   enum: IPARM_ACCESS
   
   Integer parameters tabular accessors

   IPARM_MODIFY_PARAMETER      - Indicate if parameters have been set by user             Default: API_YES             IN
   IPARM_START_TASK            - Indicate the first step to execute (see PaStiX steps)    Default: API_TASK_ORDERING   IN
   IPARM_END_TASK              - Indicate the last step to execute (see PaStiX steps)     Default: API_TASK_CLEAN      IN
   IPARM_VERBOSE               - Verbose mode (see Verbose modes)                         Default: API_VERBOSE_NO      IN
   IPARM_DOF_NBR               - Degree of freedom per node                               Default: 1                   IN
   IPARM_ITERMAX               - Maximum iteration number for refinement                  Default: 250                 IN
   IPARM_MATRIX_VERIFICATION   - Check the input matrix                                   Default: API_NO              IN
   IPARM_MC64                  - MC64 operation <pastix.h> IGNORE                         Default: 0                   IN
   IPARM_ONLY_RAFF             - Refinement only                                          Default: API_NO              IN
   IPARM_CSCD_CORRECT          - Indicate if the cscd has been redistributed after blend  Default: API_NO              IN
   IPARM_NBITER                - Number of iterations performed in refinement   	  Default: -                   OUT
   IPARM_TRACEFMT              - Trace format (see Trace modes)                           Default: API_TRACE_PICL      IN
   IPARM_GRAPHDIST             - Specify if the given graph is distributed or not         Default: API_YES             IN
   IPARM_AMALGAMATION_LEVEL    - Amalgamation level                                       Default: 5                   IN
   IPARM_ORDERING              - Choose ordering                                          Default: API_ORDER_SCOTCH    IN
   IPARM_DEFAULT_ORDERING      - Use default ordering parameters with \scotch{} or \metis{} Default: API_YES             IN
   IPARM_ORDERING_SWITCH_LEVEL - Ordering switch level    (see \scotch{} User's Guide)    Default: 120                 IN
   IPARM_ORDERING_CMIN         - Ordering cmin parameter  (see \scotch{} User's Guide)    Default: 0                   IN
   IPARM_ORDERING_CMAX         - Ordering cmax parameter  (see \scotch{} User's Guide)    Default: 100000              IN
   IPARM_ORDERING_FRAT         - Ordering frat parameter  (see \scotch{} User's Guide)    Default: 8                   IN
   IPARM_STATIC_PIVOTING       - Static pivoting                                          Default: -                   OUT
   IPARM_METIS_PFACTOR         - \metis{} pfactor                                         Default: 0                   IN
   IPARM_NNZEROS               - Number of nonzero entries in the factorized matrix       Default: -                   OUT
   IPARM_ALLOCATED_TERMS       - Maximum memory allocated for matrix terms                Default: -                   OUT
   IPARM_BASEVAL               - Baseval used for the matrix                              Default: 0                   IN
   IPARM_MIN_BLOCKSIZE         - Minimum block size                                       Default: 60                  IN
   IPARM_MAX_BLOCKSIZE         - Maximum block size                                       Default: 120                 IN
   IPARM_SCHUR                 - Schur mode                                               Default: API_NO              IN
   IPARM_ISOLATE_ZEROS         - Isolate null diagonal terms at the end of the matrix     Default: API_NO              IN
   IPARM_RHSD_CHECK            - Set to API_NO to avoid RHS redistribution                Default: API_YES             IN
   IPARM_FACTORIZATION         - Factorization mode (see Factorization modes)             Default: API_FACT_LDLT       IN  
   IPARM_NNZEROS_BLOCK_LOCAL   - Number of nonzero entries in the local block factorized matrix Default: -                   OUT
   IPARM_CPU_BY_NODE           - Number of CPUs per SMP node                              Default: 0                   IN
   IPARM_BINDTHRD              - Thread binding mode (see Thread binding modes)           Default: API_BIND_AUTO       IN
   IPARM_THREAD_NBR            - Number of threads per MPI process                        Default: 1                   IN
   IPARM_DISTRIBUTION_LEVEL    - Distribution level IGNORE                                Default:                     IN
   IPARM_LEVEL_OF_FILL         - Level of fill for incomplete factorization               Default: 1                   IN
   IPARM_IO_STRATEGY           - IO strategy (see Checkpoints modes)                      Default: API_IO_NO           IN
   IPARM_RHS_MAKING            - Right-hand-side making (see Right-hand-side modes)      Default: API_RHS_B           IN 
   IPARM_REFINEMENT            - Refinement type (see Refinement modes)                   Default: API_RAF_GMRES       IN
   IPARM_SYM                   - Symmetric matrix mode (see Symmetric modes)              Default: API_SYM_YES         IN
   IPARM_INCOMPLETE            - Incomplete factorization                                 Default: API_NO              IN
   IPARM_ABS                   - ABS (Automatic Blocksize Splitting)                      Default: API_NO              IN
   IPARM_ESP                   - ESP (Enhanced Sparse Parallelism)                        Default: API_NO              IN
   IPARM_GMRES_IM              - GMRES restart parameter                                  Default: 25                  IN
   IPARM_FREE_CSCUSER          - Free user CSC                                            Default: API_CSC_PRESERVE    IN
   IPARM_FREE_CSCPASTIX        - Free internal CSC (Use only without call to Refin. step) Default: API_CSC_PRESERVE    IN
   IPARM_OOC_LIMIT             - Out of core memory limit (Mo)                            Default: 2000                IN  
   IPARM_OOC_THREAD            - Out of core thread number IGNORE                         Default: 1                   IN
   IPARM_OOC_ID                - Out of core run ID        IGNORE                         Default: -                   OUT
   IPARM_NB_SMP_NODE_USED      - Number of SMP node used   IGNORE                         Default:                     IN
   IPARM_THREAD_COMM_MODE      - Threaded communication mode (see Communication modes)    Default: API_THCOMM_DISABLED IN
   IPARM_NB_THREAD_COMM        - Number of thread(s) for communication                    Default: 1                   IN
   IPARM_FILL_MATRIX           - Initialize matrix coefficients (for test only)  IGNORE   Default:                     IN 
   IPARM_INERTIA               - Return the inertia (symmetric matrix without pivoting)   Default: -                   OUT 
   IPARM_ESP_NBTASKS           - Return the number of tasks generated by ESP              Default: -                   OUT 
   IPARM_ESP_THRESHOLD         - Minimal block sizee to switch in ESP mode (128 * 128)    Default: 16384               IN
   IPARM_DOF_COST              - Degree of freedom for cost computation (If different from IPARM_DOF_NBR) Default: 0                    IN
   IPARM_PID                   - Pid of the first process (used for naming the log directory) Default: -1                  OUT
   IPARM_ERROR_NUMBER          - Return value                                             Default: -                   OUT
   IPARM_SIZE                  - Iparm Size                IGNORE                         Default:                     IN 
*/
enum IPARM_ACCESS {
  IPARM_MODIFY_PARAMETER        = 0, 
  IPARM_START_TASK              = 1,
  IPARM_END_TASK                = 2,
  IPARM_VERBOSE                 = 3,
  IPARM_DOF_NBR                 = 4, 
  IPARM_ITERMAX                 = 5, 
  IPARM_MATRIX_VERIFICATION     = 6,
  IPARM_MC64                    = 7, 
  IPARM_ONLY_RAFF               = 8,
  IPARM_CSCD_CORRECT            = 9,
  IPARM_NBITER                  = 10, 
  IPARM_TRACEFMT                = 11, 
  IPARM_GRAPHDIST               = 12,
  IPARM_AMALGAMATION_LEVEL      = 13,    
  IPARM_ORDERING                = 14,
  IPARM_DEFAULT_ORDERING        = 15,
  IPARM_ORDERING_SWITCH_LEVEL   = 16,
  IPARM_ORDERING_CMIN           = 17,
  IPARM_ORDERING_CMAX           = 18,
  IPARM_ORDERING_FRAT           = 19,
  IPARM_STATIC_PIVOTING         = 20,
  IPARM_METIS_PFACTOR           = 21,
  IPARM_NNZEROS                 = 22, 
  IPARM_ALLOCATED_TERMS         = 23,
  IPARM_BASEVAL                 = 24,
  IPARM_MIN_BLOCKSIZE           = 25,
  IPARM_MAX_BLOCKSIZE           = 26,
  IPARM_SCHUR                   = 27,
  IPARM_ISOLATE_ZEROS           = 28,
  IPARM_RHSD_CHECK              = 29,
  IPARM_FACTORIZATION           = 30,
  IPARM_NNZEROS_BLOCK_LOCAL     = 31,
  IPARM_CPU_BY_NODE             = 32,
  IPARM_BINDTHRD                = 33,
  IPARM_THREAD_NBR              = 34,
  IPARM_DISTRIBUTION_LEVEL      = 35, 
  IPARM_LEVEL_OF_FILL           = 36,
  IPARM_IO_STRATEGY             = 37,
  IPARM_RHS_MAKING              = 38,
  IPARM_REFINEMENT              = 39,
  IPARM_SYM                     = 40,
  IPARM_INCOMPLETE              = 41,  
  IPARM_ABS                     = 42,
  IPARM_ESP                     = 43,
  IPARM_GMRES_IM                = 44,
  IPARM_FREE_CSCUSER            = 45,
  IPARM_FREE_CSCPASTIX          = 46,
  IPARM_OOC_LIMIT               = 47,
  IPARM_OOC_THREAD              = 48,
  IPARM_OOC_ID                  = 49,
  IPARM_NB_SMP_NODE_USED        = 50,
  IPARM_THREAD_COMM_MODE        = 51,
  IPARM_NB_THREAD_COMM          = 52,
  IPARM_FILL_MATRIX             = 53,
  IPARM_INERTIA                 = 54,
  IPARM_ESP_NBTASKS             = 55,
  IPARM_ESP_THRESHOLD           = 56,
  IPARM_DOF_COST                = 57,
  IPARM_PID                     = 62,
  IPARM_ERROR_NUMBER            = 63,
  IPARM_SIZE                    = 64  /* Need to be greater or equal to 64 for backward compatibility */
};

/* Acces au tableau dparm */
/* 
   Enum: DPARM_ACCESS
   
   Floating point parameters tabular accossors

   DPARM_FILL_IN            - Fill-in                                           Default: -                OUT
   DPARM_MEM_MAX            - Maximum memory (-DMEMORY_USAGE)                   Default: -                OUT
   DPARM_EPSILON_REFINEMENT - Epsilon for refinement                            Default: 1e^{-12}         IN
   DPARM_RELATIVE_ERROR     - Relative backward error                           Default: -                OUT
   DPARM_EPSILON_MAGN_CTRL  - Epsilon for magnitude control                     Default: 1e^{-31}         IN
   DPARM_ANALYZE_TIME       - Time for Analyse step (wallclock)                 Default: -                OUT
   DPARM_PRED_FACT_TIME     - Predicted factorization time                      Default: -                OUT
   DPARM_FACT_TIME          - Time for Numerical Factorization step (wallclock) Default: -                OUT
   DPARM_SOLV_TIME          - Time for Solve step (wallclock)                   Default: -                OUT
   DPARM_FACT_FLOPS         - Numerical Factorization flops (rate!)             Default: -                OUT
   DPARM_SOLV_FLOPS         - Solve flops (rate!)                               Default: -                OUT
   DPARM_RAFF_TIME          - Time for Refinement step (wallclock)              Default: -                OUT
   DPARM_SIZE               - Dparm Size         IGNORE                         Default: -                IN 
 */
enum DPARM_ACCESS {
  DPARM_FILL_IN                 = 1,
  DPARM_MEM_MAX                 = 2,
  DPARM_EPSILON_REFINEMENT      = 5,
  DPARM_RELATIVE_ERROR          = 6, 
  DPARM_EPSILON_MAGN_CTRL       = 10,
  DPARM_ANALYZE_TIME            = 18,
  DPARM_PRED_FACT_TIME          = 19,
  DPARM_FACT_TIME               = 20,
  DPARM_SOLV_TIME               = 21,
  DPARM_FACT_FLOPS              = 22,
  DPARM_SOLV_FLOPS              = 23,
  DPARM_RAFF_TIME               = 24, 
  DPARM_SIZE                    = 64 /* Need to be greater or equal to 64 for backward compatibility */
};

/** Etapes de résolution de PaStiX */
/*
  Enum: API_TASK

  PaStiX step modes (index IPARM_START_TASK and IPARM_END_TASK)

  API_TASK_INIT       - Set default parameters
  API_TASK_ORDERING   -	Ordering         
  API_TASK_SYMBFACT   -	Symbolic factorization
  API_TASK_ANALYSE    -	Tasks mapping and scheduling
  API_TASK_NUMFACT    -	Numerical factorization   
  API_TASK_SOLVE      -	Numerical solve        
  API_TASK_REFINE     -	Numerical refinement
  API_TASK_CLEAN      -	Clean           
 */
/* _POS_ 1 */
enum API_TASK {
  API_TASK_INIT       = 0,
  API_TASK_ORDERING   = 1, 
  API_TASK_SYMBFACT   = 2, 
  API_TASK_ANALYSE    = 3, 
  API_TASK_NUMFACT    = 4, 
  API_TASK_SOLVE      = 5, 
  API_TASK_REFINE     = 6, 
  API_TASK_CLEAN      = 7  
};

/** Etapes de résolution de PaStiX pour compatibilte avec les anciennes version */
/*
  Enum: API_TASK_OLD

  API_TASK_SCOTCH     -	Ordering         
  API_TASK_FAX        -	Symbolic factorization
  API_TASK_BLEND      -	Tasks mapping and scheduling
  API_TASK_SOPALIN    -	Numerical factorization   
  API_TASK_UPDOWN     -	Numerical solve        
  API_TASK_REFINEMENT -	Numerical refinement
 */
/* _POS_ -1 */
enum API_TASK_OLD {
  API_TASK_SCOTCH     = 1,
  API_TASK_FAX        = 2,
  API_TASK_BLEND      = 3,
  API_TASK_SOPALIN    = 4,
  API_TASK_UPDOWN     = 5,
  API_TASK_REFINEMENT = 6
};

/** Affichage de PaStiX */
/* 
   Enum: API_VERBOSE

   Verbose modes (index IPARM_VERBOSE)

   API_VERBOSE_NOT        - Silent mode, no messages   
   API_VERBOSE_NO         - Some messages
   API_VERBOSE_YES        - Many messages
   API_VERBOSE_CHATTERBOX - Like a gossip
   API_VERBOSE_UNBEARABLE - Really talking too much...
*/
/* _POS_ 5 */
enum API_VERBOSE {
  API_VERBOSE_NOT        = 0, /* Nothing  */
  API_VERBOSE_NO         = 1, /* Default  */
  API_VERBOSE_YES        = 2, /* Extended */
  API_VERBOSE_CHATTERBOX = 3,
  API_VERBOSE_UNBEARABLE = 4
};

/** Load strategy for graph and ordering */
/*
  Enum: API_IO

  Check-points modes (index IPARM_IO)
  
  API_IO_NO         - No output or input   
  API_IO_LOAD       - Load ordering during ordering step and symbol matrix instead of symbolic factorisation.
  API_IO_SAVE       - Save ordering during ordering step and symbol matrix instead of symbolic factorisation.
  API_IO_LOAD_GRAPH - Load graph during ordering step.
  API_IO_SAVE_GRAPH - Save graph during ordering step.
  API_IO_LOAD_CSC   - Load CSC(d) during ordering step.
  API_IO_SAVE_CSC   - Save CSC(d) during ordering step.
 */
/* _POS_ 6 */
enum API_IO {
  API_IO_NO         = 0, 
  API_IO_LOAD       = 1,
  API_IO_SAVE       = 2,
  API_IO_LOAD_GRAPH = 4,
  API_IO_SAVE_GRAPH = 8,
  API_IO_LOAD_CSC   = 16,
  API_IO_SAVE_CSC   = 32
};

/** Genération du second membre */
/*
  Enum: API_RHS

  Right-hand-side modes (index IPARM_RHS)

  API_RHS_B - User's right hand side
  API_RHS_1 - $ \forall i, X_i = 1 $
  API_RHS_I - $ \forall i, X_i = i $

 */
/* _POS_ 7 */
enum API_RHS {
  API_RHS_B = 0, /* Utilisation du second membre fournit */
  API_RHS_1 = 1, /* Utilisation d'un second membre dont tous les coefficients valent 1 */
  API_RHS_I = 2  /* Utilisation d'un second membre tel que RHS(i) = i */
};

/** Type de raffinement utilisé */
/*
  Enum: API_RAF
  
  Refinement modes (index IPARM_REFINEMENT)

  API_RAF_GMRES - GMRES        
  API_RAF_GRAD  - Conjugate Gradient ($LL^t$ or $LDL^t$ factorization)
  API_RAF_PIVOT - Iterative Refinement (only for $LU$ factorization)
  
 */
/* _POS_ 8 */
enum API_RAF {
  API_RAF_GMRES = 0, /* Utilisation de GMRES */
  API_RAF_GRAD  = 1, /* Utilisation du gradient conjugué */
  API_RAF_PIVOT = 1  /* Utilisation de la méthode du pivot */
};

/** Type de facto utilisée (LLT,LDLT,LU)*/
/*
  Enum: API_FACT

  Factorization modes (index IPARM_FACTORISATION_TYPE)

  API_FACT_LLT  - $LL^t$ Factorization 
  API_FACT_LDLT - $LDL^t$ Factorization
  API_FACT_LU   - $LU$ Factorization

*/
/* _POS_ 4 */
enum API_FACT {
  API_FACT_LLT  = 0, /* Factorisation de Cholesky */
  API_FACT_LDLT = 1, /* Factorisation de Crout */
  API_FACT_LU   = 2  /* Factorisation LU */
};

/** Matrice symétrique ou non (0 : symétrique, 1 : non) */
/*
  Enum: API_SYM

  Symetric modes (index IPARM_SYM)

  API_SYM_YES - Symmetric matrix
  API_SYM_NO  - Nonsymmetric matrix

 */
/* _POS_ 3 */
enum API_SYM {
  API_SYM_YES = 0, /* Matrice symetrique     */
  API_SYM_NO  = 1  /* Matrice non symetrique */
};

/** Supressing user CSC(D) when not usefull anymore */
/*
  Enum: API_ERASE_CSC

  CSC Management modes (index IPARM_FREE_CSCUSER and IPARM_FREE_CSCPASTIX)

  API_CSC_PRESERVE - Do not free the CSC
  API_CSC_FREE     - Free the CSC when not needed anymore
 */
/* _POS_ 11 */
enum API_ERASE_CSC{
  API_CSC_PRESERVE = 0,
  API_CSC_FREE     = 1
};

/** DMP communication mode */
/*
  Enum: API_COMM_THREAD_MODE

  Comunication modes (index IPARM_NB_THREAD_COMM)

  API_THCOMM_DISABLED - No dedicated communication thread        
  API_THCOMM_ONE      -	One dedicated communication thread
  API_THCOMM_DEFINED  -	Given by IPARM_NB_THREAD_COMM
  API_THCOMM_NBPROC   -	One communication thread per computation thread
 */
/* _POS_ 9 */
enum API_COMM_THREAD_MODE {
  API_THCOMM_DISABLED = 0, /* Chaque Thread s'ocupe de ses communications                 */
  API_THCOMM_ONE      = 1, /* Un seul thread de communication                             */
  API_THCOMM_DEFINED  = 2, /* Nombre de threads de communication paramétrable             */
  API_THCOMM_NBPROC   = 3  /* Autant de threads de communication que de threads de calcul */
};

/** Thread binding */
/*
  Enum: API_BIND_MODE

  Thread-binding modes (index IPARM_BINTHRD)

  API_BIND_NO   - Do not bind thread
  API_BIND_AUTO	- Default binding                            
  API_BIND_TAB  - Use vector given by pastix_setBind
*/
/* _POS_ 12 */
enum API_BIND_MODE {
  API_BIND_NO      = 0, /* Do not bind threads                            */
  API_BIND_AUTO    = 1, /* Default thread binding                         */
  API_BIND_TAB     = 2  /* Use tabular given by pastix_setBind to bind */
};

/** Boolean */
/*
  Enum: API_BOOLEAN

  Boolean modes (All boolean except IPARM_SYM)

  API_NO  - No
  API_YES - Yes
 */
/* _POS_ 2 */
enum API_BOOLEAN {
  API_NO  = 0,
  API_YES = 1
};

/** Trace format */
/*
  Enum: API_TRACEFMT

  Trace modes (index IPARM_TRACEFMT)

  API_TRACE_PICL       - Use PICL trace format
  API_TRACE_PAJE       - Use Paje trace format
  API_TRACE_HUMREAD    - Use human-readable text trace format
  API_TRACE_UNFORMATED - Unformated trace format
 */
/* _POS_ 10 */
enum API_TRACEFMT {
  API_TRACE_PICL       = 0, /* Use PICL trace format       */
  API_TRACE_PAJE       = 1, /* Use Paje trace format       */
  API_TRACE_HUMREAD    = 2, /* Use text trace format       */
  API_TRACE_UNFORMATED = 3  /* Use unformated trace format */
};

/*
  Enum: API_ORDER

  Ordering modes (index IPARM_ORDERING)

  API_ORDER_SCOTCH   - Use \scotch{} ordering
  API_ORDER_METIS    - Use \metis{} ordering
  API_ORDER_PERSONAL - Apply user's permutation (resp. reverse permutation)
  API_ORDER_LOAD     - Load ordering from disk
 */
/* _POS_ 11 */
enum API_ORDER {
  API_ORDER_SCOTCH    = 0,
  API_ORDER_METIS     = 1,
  API_ORDER_PERSONAL  = 2,
  API_ORDER_LOAD      = 3
};


/*
  Enum: MODULES

  Module Identification number.

  If an error occurs, error value is set to 
  MODULE + EER_NUMBER.

  User can catch error by computing iparm[IPARM_ERROR_NUMBER]%100.

  MODULE can be catch by computing iparm[IPARM_ERROR_NUMBER] - iparm[IPARM_ERROR_NUMBER]%100.
  
  MOD_UNKNOWN - Unknown module
  MOD_SOPALIN - Numerical factorisation module
  MOD_BLEND   - Analysing module
  MOD_SCOTCH  - Scotch module 
  MOD_FAX     - Symbolic factorisation module
  MOD_ORDER   - Order module
  MOD_COMMON  - Common module
  MOD_SI      - 
  MOD_GRAPH   - Graph module
  MOD_SYMBOL  - Symbol structure module
  MOD_KASS    - Kass module
  MOD_BUBBLE  - Bubble
  MOD_MURGE   - Murge

*/
enum MODULES {
  MOD_UNKNOWN =    0,
  MOD_SOPALIN =  100,
  MOD_BLEND   =  200,
  MOD_SCOTCH  =  300,
  MOD_FAX     =  400,
  MOD_ORDER   =  500,
  MOD_COMMON  =  600,
  MOD_SI      =  700,
  MOD_GRAPH   =  800,
  MOD_SYMBOL  =  900,
  MOD_KASS    = 1000,
  MOD_BUBBLE  = 1100,
  MOD_MURGE   = 1200
};

/* Enum: ERR_NUMBERS

   Error Numbers 

   NO_ERR             - No error
   UNKNOWN_ERR        - Unknown error
   ALLOC_ERR          - Allocation error
   ASSERT_ERR         - Error in one assertion
   NOTIMPLEMENTED_ERR - Not implemented feature
   OUTOFMEMORY_ERR    - Not enough memory (OOC)
   THREAD_ERR         - Error with threads
   INTERNAL_ERR       - Internal error
   BADPARAMETER_ERR   - Bad parameters given
   FILE_ERR           - Error in In/Out operations
   BAD_DEFINE_ERROR   - Error with defines during compilation
*/
/* Need to conserve it MURGE compliant */
enum ERR_NUMBERS {
  NO_ERR             = 0,
  UNKNOWN_ERR        = 1,
  ALLOC_ERR          = 2, 
  ASSERT_ERR         = 3,
  NOTIMPLEMENTED_ERR = 4,
  OUTOFMEMORY_ERR    = 5,
  THREAD_ERR         = 6,
  INTERNAL_ERR       = 7,
  BADPARAMETER_ERR   = 8,
  FILE_ERR           = 9,
  BAD_DEFINE_ERR     = 10,
  INTEGER_TYPE_ERR   = 11,
  IO_ERR             = 12,
  MATRIX_ERR         = 13,
  FLOAT_TYPE_ERR     = 14,
  STEP_ORDER_ERR     = 15
};

/** Matrix type */
#define MTX_ISSYM(a) ((a)[1]=='S')
#define MTX_ISCOM(a) ((a)[0]=='C')
#define MTX_ISRHX(a) ((a)[2]=='X')
#define MTX_ISRHS(a) ((a)[0]!='\0')

/* **************************************** */
